/*******************************************************************************
* File Name: j1FreeRTOS_StatsCounter.c  
* Version 3.0
*
*  Description:
*     The Counter component consists of a 8, 16, 24 or 32-bit counter with
*     a selectable period between 2 and 2^Width - 1.  
*
*   Note:
*     None
*
********************************************************************************
* Copyright 2008-2012, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "j1FreeRTOS_StatsCounter.h"

uint8 j1FreeRTOS_StatsCounter_initVar = 0u;


/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_Init
********************************************************************************
* Summary:
*     Initialize to the schematic state
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
*******************************************************************************/
void j1FreeRTOS_StatsCounter_Init(void) 
{
        #if (!j1FreeRTOS_StatsCounter_UsingFixedFunction && !j1FreeRTOS_StatsCounter_ControlRegRemoved)
            uint8 ctrl;
        #endif /* (!j1FreeRTOS_StatsCounter_UsingFixedFunction && !j1FreeRTOS_StatsCounter_ControlRegRemoved) */
        
        #if(!j1FreeRTOS_StatsCounter_UsingFixedFunction) 
            /* Interrupt State Backup for Critical Region*/
            uint8 j1FreeRTOS_StatsCounter_interruptState;
        #endif /* (!j1FreeRTOS_StatsCounter_UsingFixedFunction) */
        
        #if (j1FreeRTOS_StatsCounter_UsingFixedFunction)
            /* Clear all bits but the enable bit (if it's already set for Timer operation */
            j1FreeRTOS_StatsCounter_CONTROL &= j1FreeRTOS_StatsCounter_CTRL_ENABLE;
            
            /* Clear the mode bits for continuous run mode */
            #if (CY_PSOC5A)
                j1FreeRTOS_StatsCounter_CONTROL2 &= ((uint8)(~j1FreeRTOS_StatsCounter_CTRL_MODE_MASK));
            #endif /* (CY_PSOC5A) */
            #if (CY_PSOC3 || CY_PSOC5LP)
                j1FreeRTOS_StatsCounter_CONTROL3 &= ((uint8)(~j1FreeRTOS_StatsCounter_CTRL_MODE_MASK));                
            #endif /* (CY_PSOC3 || CY_PSOC5LP) */
            /* Check if One Shot mode is enabled i.e. RunMode !=0*/
            #if (j1FreeRTOS_StatsCounter_RunModeUsed != 0x0u)
                /* Set 3rd bit of Control register to enable one shot mode */
                j1FreeRTOS_StatsCounter_CONTROL |= j1FreeRTOS_StatsCounter_ONESHOT;
            #endif /* (j1FreeRTOS_StatsCounter_RunModeUsed != 0x0u) */
            
            /* Set the IRQ to use the status register interrupts */
            j1FreeRTOS_StatsCounter_CONTROL2 |= j1FreeRTOS_StatsCounter_CTRL2_IRQ_SEL;
            
            /* Clear and Set SYNCTC and SYNCCMP bits of RT1 register */
            j1FreeRTOS_StatsCounter_RT1 &= ((uint8)(~j1FreeRTOS_StatsCounter_RT1_MASK));
            j1FreeRTOS_StatsCounter_RT1 |= j1FreeRTOS_StatsCounter_SYNC;     
                    
            /*Enable DSI Sync all all inputs of the Timer*/
            j1FreeRTOS_StatsCounter_RT1 &= ((uint8)(~j1FreeRTOS_StatsCounter_SYNCDSI_MASK));
            j1FreeRTOS_StatsCounter_RT1 |= j1FreeRTOS_StatsCounter_SYNCDSI_EN;

        #else
            #if(!j1FreeRTOS_StatsCounter_ControlRegRemoved)
            /* Set the default compare mode defined in the parameter */
            ctrl = j1FreeRTOS_StatsCounter_CONTROL & ((uint8)(~j1FreeRTOS_StatsCounter_CTRL_CMPMODE_MASK));
            j1FreeRTOS_StatsCounter_CONTROL = ctrl | j1FreeRTOS_StatsCounter_DEFAULT_COMPARE_MODE;
            
            /* Set the default capture mode defined in the parameter */
            ctrl = j1FreeRTOS_StatsCounter_CONTROL & ((uint8)(~j1FreeRTOS_StatsCounter_CTRL_CAPMODE_MASK));
            
            #if( 0 != j1FreeRTOS_StatsCounter_CAPTURE_MODE_CONF)
                j1FreeRTOS_StatsCounter_CONTROL = ctrl | j1FreeRTOS_StatsCounter_DEFAULT_CAPTURE_MODE;
            #else
                j1FreeRTOS_StatsCounter_CONTROL = ctrl;
            #endif /* 0 != j1FreeRTOS_StatsCounter_CAPTURE_MODE */ 
            
            #endif /* (!j1FreeRTOS_StatsCounter_ControlRegRemoved) */
        #endif /* (j1FreeRTOS_StatsCounter_UsingFixedFunction) */
        
        /* Clear all data in the FIFO's */
        #if (!j1FreeRTOS_StatsCounter_UsingFixedFunction)
            j1FreeRTOS_StatsCounter_ClearFIFO();
        #endif /* (!j1FreeRTOS_StatsCounter_UsingFixedFunction) */
        
        /* Set Initial values from Configuration */
        j1FreeRTOS_StatsCounter_WritePeriod(j1FreeRTOS_StatsCounter_INIT_PERIOD_VALUE);
        #if (!(j1FreeRTOS_StatsCounter_UsingFixedFunction && (CY_PSOC5A)))
            j1FreeRTOS_StatsCounter_WriteCounter(j1FreeRTOS_StatsCounter_INIT_COUNTER_VALUE);
        #endif /* (!(j1FreeRTOS_StatsCounter_UsingFixedFunction && (CY_PSOC5A))) */
        j1FreeRTOS_StatsCounter_SetInterruptMode(j1FreeRTOS_StatsCounter_INIT_INTERRUPTS_MASK);
        
        #if (!j1FreeRTOS_StatsCounter_UsingFixedFunction)
            /* Read the status register to clear the unwanted interrupts */
            (void)j1FreeRTOS_StatsCounter_ReadStatusRegister();
            /* Set the compare value (only available to non-fixed function implementation */
            j1FreeRTOS_StatsCounter_WriteCompare(j1FreeRTOS_StatsCounter_INIT_COMPARE_VALUE);
            /* Use the interrupt output of the status register for IRQ output */
            
            /* CyEnterCriticalRegion and CyExitCriticalRegion are used to mark following region critical*/
            /* Enter Critical Region*/
            j1FreeRTOS_StatsCounter_interruptState = CyEnterCriticalSection();
            
            j1FreeRTOS_StatsCounter_STATUS_AUX_CTRL |= j1FreeRTOS_StatsCounter_STATUS_ACTL_INT_EN_MASK;
            
            /* Exit Critical Region*/
            CyExitCriticalSection(j1FreeRTOS_StatsCounter_interruptState);
            
        #endif /* (!j1FreeRTOS_StatsCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_Enable
********************************************************************************
* Summary:
*     Enable the Counter
* 
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: 
*   If the Enable mode is set to Hardware only then this function has no effect 
*   on the operation of the counter.
*
*******************************************************************************/
void j1FreeRTOS_StatsCounter_Enable(void) 
{
    /* Globally Enable the Fixed Function Block chosen */
    #if (j1FreeRTOS_StatsCounter_UsingFixedFunction)
        j1FreeRTOS_StatsCounter_GLOBAL_ENABLE |= j1FreeRTOS_StatsCounter_BLOCK_EN_MASK;
        j1FreeRTOS_StatsCounter_GLOBAL_STBY_ENABLE |= j1FreeRTOS_StatsCounter_BLOCK_STBY_EN_MASK;
    #endif /* (j1FreeRTOS_StatsCounter_UsingFixedFunction) */  
        
    /* Enable the counter from the control register  */
    /* If Fixed Function then make sure Mode is set correctly */
    /* else make sure reset is clear */
    #if(!j1FreeRTOS_StatsCounter_ControlRegRemoved || j1FreeRTOS_StatsCounter_UsingFixedFunction)
        j1FreeRTOS_StatsCounter_CONTROL |= j1FreeRTOS_StatsCounter_CTRL_ENABLE;                
    #endif /* (!j1FreeRTOS_StatsCounter_ControlRegRemoved || j1FreeRTOS_StatsCounter_UsingFixedFunction) */
    
}


/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_Start
********************************************************************************
* Summary:
*  Enables the counter for operation 
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Global variables:
*  j1FreeRTOS_StatsCounter_initVar: Is modified when this function is called for the  
*   first time. Is used to ensure that initialization happens only once.
*
*******************************************************************************/
void j1FreeRTOS_StatsCounter_Start(void) 
{
    if(j1FreeRTOS_StatsCounter_initVar == 0u)
    {
        j1FreeRTOS_StatsCounter_Init();
        
        j1FreeRTOS_StatsCounter_initVar = 1u; /* Clear this bit for Initialization */        
    }
    
    /* Enable the Counter */
    j1FreeRTOS_StatsCounter_Enable();        
}


/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_Stop
********************************************************************************
* Summary:
* Halts the counter, but does not change any modes or disable interrupts.
*
* Parameters:  
*  void  
*
* Return: 
*  void
*
* Side Effects: If the Enable mode is set to Hardware only then this function
*               has no effect on the operation of the counter.
*
*******************************************************************************/
void j1FreeRTOS_StatsCounter_Stop(void) 
{
    /* Disable Counter */
    #if(!j1FreeRTOS_StatsCounter_ControlRegRemoved || j1FreeRTOS_StatsCounter_UsingFixedFunction)
        j1FreeRTOS_StatsCounter_CONTROL &= ((uint8)(~j1FreeRTOS_StatsCounter_CTRL_ENABLE));        
    #endif /* (!j1FreeRTOS_StatsCounter_ControlRegRemoved || j1FreeRTOS_StatsCounter_UsingFixedFunction) */
    
    /* Globally disable the Fixed Function Block chosen */
    #if (j1FreeRTOS_StatsCounter_UsingFixedFunction)
        j1FreeRTOS_StatsCounter_GLOBAL_ENABLE &= ((uint8)(~j1FreeRTOS_StatsCounter_BLOCK_EN_MASK));
        j1FreeRTOS_StatsCounter_GLOBAL_STBY_ENABLE &= ((uint8)(~j1FreeRTOS_StatsCounter_BLOCK_STBY_EN_MASK));
    #endif /* (j1FreeRTOS_StatsCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_SetInterruptMode
********************************************************************************
* Summary:
* Configures which interrupt sources are enabled to generate the final interrupt
*
* Parameters:  
*  InterruptsMask: This parameter is an or'd collection of the status bits
*                   which will be allowed to generate the counters interrupt.   
*
* Return: 
*  void
*
*******************************************************************************/
void j1FreeRTOS_StatsCounter_SetInterruptMode(uint8 interruptsMask) 
{
    j1FreeRTOS_StatsCounter_STATUS_MASK = interruptsMask;
}


/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_ReadStatusRegister
********************************************************************************
* Summary:
*   Reads the status register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the status register
*
* Side Effects:
*   Status register bits may be clear on read. 
*
*******************************************************************************/
uint8   j1FreeRTOS_StatsCounter_ReadStatusRegister(void) 
{
    return j1FreeRTOS_StatsCounter_STATUS;
}


#if(!j1FreeRTOS_StatsCounter_ControlRegRemoved)
/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_ReadControlRegister
********************************************************************************
* Summary:
*   Reads the control register and returns it's state. This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
uint8   j1FreeRTOS_StatsCounter_ReadControlRegister(void) 
{
    return j1FreeRTOS_StatsCounter_CONTROL;
}


/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_WriteControlRegister
********************************************************************************
* Summary:
*   Sets the bit-field of the control register.  This function should use
*       defined types for the bit-field information as the bits in this
*       register may be permuteable.
*
* Parameters:  
*  void
*
* Return: 
*  (uint8) The contents of the control register
*
*******************************************************************************/
void    j1FreeRTOS_StatsCounter_WriteControlRegister(uint8 control) 
{
    j1FreeRTOS_StatsCounter_CONTROL = control;
}

#endif  /* (!j1FreeRTOS_StatsCounter_ControlRegRemoved) */


#if (!(j1FreeRTOS_StatsCounter_UsingFixedFunction && (CY_PSOC5A)))
/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_WriteCounter
********************************************************************************
* Summary:
*   This funtion is used to set the counter to a specific value
*
* Parameters:  
*  counter:  New counter value. 
*
* Return: 
*  void 
*
*******************************************************************************/
void j1FreeRTOS_StatsCounter_WriteCounter(uint32 counter) \
                                   
{
    #if(j1FreeRTOS_StatsCounter_UsingFixedFunction)
        /* assert if block is already enabled */
        CYASSERT (0u == (j1FreeRTOS_StatsCounter_GLOBAL_ENABLE & j1FreeRTOS_StatsCounter_BLOCK_EN_MASK));
        /* If block is disabled, enable it and then write the counter */
        j1FreeRTOS_StatsCounter_GLOBAL_ENABLE |= j1FreeRTOS_StatsCounter_BLOCK_EN_MASK;
        CY_SET_REG16(j1FreeRTOS_StatsCounter_COUNTER_LSB_PTR, (uint16)counter);
        j1FreeRTOS_StatsCounter_GLOBAL_ENABLE &= ((uint8)(~j1FreeRTOS_StatsCounter_BLOCK_EN_MASK));
    #else
        CY_SET_REG32(j1FreeRTOS_StatsCounter_COUNTER_LSB_PTR, counter);
    #endif /* (j1FreeRTOS_StatsCounter_UsingFixedFunction) */
}
#endif /* (!(j1FreeRTOS_StatsCounter_UsingFixedFunction && (CY_PSOC5A))) */


/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_ReadCounter
********************************************************************************
* Summary:
* Returns the current value of the counter.  It doesn't matter
* if the counter is enabled or running.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint32) The present value of the counter.
*
*******************************************************************************/
uint32 j1FreeRTOS_StatsCounter_ReadCounter(void) 
{
    /* Force capture by reading Accumulator */
    /* Must first do a software capture to be able to read the counter */
    /* It is up to the user code to make sure there isn't already captured data in the FIFO */
    #if(j1FreeRTOS_StatsCounter_UsingFixedFunction)
		(void)CY_GET_REG16(j1FreeRTOS_StatsCounter_COUNTER_LSB_PTR);
	#else
		(void)CY_GET_REG8(j1FreeRTOS_StatsCounter_COUNTER_LSB_PTR_8BIT);
	#endif/* (j1FreeRTOS_StatsCounter_UsingFixedFunction) */
    
    /* Read the data from the FIFO (or capture register for Fixed Function)*/
    #if(j1FreeRTOS_StatsCounter_UsingFixedFunction)
        return ((uint32)CY_GET_REG16(j1FreeRTOS_StatsCounter_STATICCOUNT_LSB_PTR));
    #else
        return (CY_GET_REG32(j1FreeRTOS_StatsCounter_STATICCOUNT_LSB_PTR));
    #endif /* (j1FreeRTOS_StatsCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_ReadCapture
********************************************************************************
* Summary:
*   This function returns the last value captured.
*
* Parameters:  
*  void
*
* Return: 
*  (uint32) Present Capture value.
*
*******************************************************************************/
uint32 j1FreeRTOS_StatsCounter_ReadCapture(void) 
{
    #if(j1FreeRTOS_StatsCounter_UsingFixedFunction)
        return ((uint32)CY_GET_REG16(j1FreeRTOS_StatsCounter_STATICCOUNT_LSB_PTR));
    #else
        return (CY_GET_REG32(j1FreeRTOS_StatsCounter_STATICCOUNT_LSB_PTR));
    #endif /* (j1FreeRTOS_StatsCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_WritePeriod
********************************************************************************
* Summary:
* Changes the period of the counter.  The new period 
* will be loaded the next time terminal count is detected.
*
* Parameters:  
*  period: (uint32) A value of 0 will result in
*         the counter remaining at zero.  
*
* Return: 
*  void
*
*******************************************************************************/
void j1FreeRTOS_StatsCounter_WritePeriod(uint32 period) 
{
    #if(j1FreeRTOS_StatsCounter_UsingFixedFunction)
        CY_SET_REG16(j1FreeRTOS_StatsCounter_PERIOD_LSB_PTR,(uint16)period);
    #else
        CY_SET_REG32(j1FreeRTOS_StatsCounter_PERIOD_LSB_PTR, period);
    #endif /* (j1FreeRTOS_StatsCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_ReadPeriod
********************************************************************************
* Summary:
* Reads the current period value without affecting counter operation.
*
* Parameters:  
*  void:  
*
* Return: 
*  (uint32) Present period value.
*
*******************************************************************************/
uint32 j1FreeRTOS_StatsCounter_ReadPeriod(void) 
{
    #if(j1FreeRTOS_StatsCounter_UsingFixedFunction)
        return ((uint32)CY_GET_REG16(j1FreeRTOS_StatsCounter_PERIOD_LSB_PTR));
    #else
        return (CY_GET_REG32(j1FreeRTOS_StatsCounter_PERIOD_LSB_PTR));
    #endif /* (j1FreeRTOS_StatsCounter_UsingFixedFunction) */
}


#if (!j1FreeRTOS_StatsCounter_UsingFixedFunction)
/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_WriteCompare
********************************************************************************
* Summary:
* Changes the compare value.  The compare output will 
* reflect the new value on the next UDB clock.  The compare output will be 
* driven high when the present counter value compares true based on the 
* configured compare mode setting. 
*
* Parameters:  
*  Compare:  New compare value. 
*
* Return: 
*  void
*
*******************************************************************************/
void j1FreeRTOS_StatsCounter_WriteCompare(uint32 compare) \
                                   
{
    #if(j1FreeRTOS_StatsCounter_UsingFixedFunction)
        CY_SET_REG16(j1FreeRTOS_StatsCounter_COMPARE_LSB_PTR, (uint16)compare);
    #else
        CY_SET_REG32(j1FreeRTOS_StatsCounter_COMPARE_LSB_PTR, compare);
    #endif /* (j1FreeRTOS_StatsCounter_UsingFixedFunction) */
}


/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_ReadCompare
********************************************************************************
* Summary:
* Returns the compare value.
*
* Parameters:  
*  void:
*
* Return: 
*  (uint32) Present compare value.
*
*******************************************************************************/
uint32 j1FreeRTOS_StatsCounter_ReadCompare(void) 
{
    return (CY_GET_REG32(j1FreeRTOS_StatsCounter_COMPARE_LSB_PTR));
}


#if (j1FreeRTOS_StatsCounter_COMPARE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_SetCompareMode
********************************************************************************
* Summary:
*  Sets the software controlled Compare Mode.
*
* Parameters:
*  compareMode:  Compare Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void j1FreeRTOS_StatsCounter_SetCompareMode(uint8 compareMode) 
{
    /* Clear the compare mode bits in the control register */
    j1FreeRTOS_StatsCounter_CONTROL &= ((uint8)(~j1FreeRTOS_StatsCounter_CTRL_CMPMODE_MASK));
    
    /* Write the new setting */
    j1FreeRTOS_StatsCounter_CONTROL |= compareMode;
}
#endif  /* (j1FreeRTOS_StatsCounter_COMPARE_MODE_SOFTWARE) */


#if (j1FreeRTOS_StatsCounter_CAPTURE_MODE_SOFTWARE)
/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_SetCaptureMode
********************************************************************************
* Summary:
*  Sets the software controlled Capture Mode.
*
* Parameters:
*  captureMode:  Capture Mode Enumerated Type.
*
* Return:
*  void
*
*******************************************************************************/
void j1FreeRTOS_StatsCounter_SetCaptureMode(uint8 captureMode) 
{
    /* Clear the capture mode bits in the control register */
    j1FreeRTOS_StatsCounter_CONTROL &= ((uint8)(~j1FreeRTOS_StatsCounter_CTRL_CAPMODE_MASK));
    
    /* Write the new setting */
    j1FreeRTOS_StatsCounter_CONTROL |= ((uint8)((uint8)captureMode << j1FreeRTOS_StatsCounter_CTRL_CAPMODE0_SHIFT));
}
#endif  /* (j1FreeRTOS_StatsCounter_CAPTURE_MODE_SOFTWARE) */


/*******************************************************************************
* Function Name: j1FreeRTOS_StatsCounter_ClearFIFO
********************************************************************************
* Summary:
*   This function clears all capture data from the capture FIFO
*
* Parameters:  
*  void:
*
* Return: 
*  None
*
*******************************************************************************/
void j1FreeRTOS_StatsCounter_ClearFIFO(void) 
{

    while(0u != (j1FreeRTOS_StatsCounter_ReadStatusRegister() & j1FreeRTOS_StatsCounter_STATUS_FIFONEMP))
    {
        (void)j1FreeRTOS_StatsCounter_ReadCapture();
    }

}
#endif  /* (!j1FreeRTOS_StatsCounter_UsingFixedFunction) */


/* [] END OF FILE */

