//=====================================================================
//
//	j1GFX.h - j1sys device independent graphics lib
//
//	loosely based on a re-write of the adafruit graphics library
//
//	version 1.1 - 15 feb 2014
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

#if !defined(j1GFX_H)
#define j1GFX_H
	
//---------------------------------------------------------------------
//	our instance parameters made into defines
//---------------------------------------------------------------------
	
#define	j1GFX_PARAM_FONT_5X7		1
#define	j1GFX_PARAM_FONT_8X12		1

#define	j1GFX_PARAM_NUM_5X7			0
#define	j1GFX_PARAM_NUM_8X12		0
	
#define	j1GFX_PARAM_ROTATES			0
	
#define	j1GFX_PARAM_SDL_TEXT		1
//#define	j1GFX_PARAM_SDL_TEXT_CENTER	
//#define	j1GFX_PARAM_SDL_TEXT_BUTTON	
	
//---------------------------------------------------------------------
//	processor dependent includes
//---------------------------------------------------------------------
	
#include <cytypes.h>

//---------------------------------------------------------------------
//	stdlib includes
//---------------------------------------------------------------------

#include <stdbool.h>
#include <string.h>
	
//---------------------------------------------------------------------
//	public font tables
//---------------------------------------------------------------------

#if j1GFX_PARAM_FONT_5X7 || j1GFX_PARAM_NUM_5X7
extern	const uint8	j1GFX_Font5x7[];
#endif

#if j1GFX_PARAM_FONT_8X12 || j1GFX_PARAM_NUM_8X12
extern	const uint8	j1GFX_Font8x12[];
#endif
	
//---------------------------------------------------------------------
//	j1GFX_CTRL
//---------------------------------------------------------------------
	
typedef enum
{
	j1GFX_ROTATE_N,
#if j1GFX_PARAM_ROTATES	
	j1GFX_ROTATE_E,
	j1GFX_ROTATE_S,
	j1GFX_ROTATE_W
#endif
} j1GFX_ROTATE;
	
typedef enum
{
#if j1GFX_PARAM_FONT_5X7
	j1GFX_FONT_5X7,
#else
	j1GFX_FONT_5X7_REMOVED,
#endif
#if j1GFX_PARAM_FONT_8X12
	j1GFX_FONT_8X12,
#else
	j1GFX_FONT_8X12_REMOVED,
#endif
} j1GFX_FONT;
	

typedef struct _j1GFX_CTRL
{
	void			*hardware;			// hardware control struct pointer
	
	uint16			displayWidth;		// display width
	uint16			displayHeight;		// display height
	
	uint16			regWidth;			// region width
	uint16			regHeight;			// region height
	uint16			regOffsetX;			// region offset x
	uint16			regOffsetY;			// region offset y
	
	j1GFX_FONT		textFont;			// text font size
	uint8			textScale;			// text current font scale
	uint16			textFontW;			// text current font width
	uint16			textFontH;			// text current font height
	uint32			textFG;				// text current fg (RGB)
	uint32			textBG;				// text current bg (RGB)
	uint16			textX;				// text current x
	uint16			textY;				// text current y
	
} j1GFX_CTRL;

typedef struct _j1GFX_HOOKS
{
	void	(*SetRotation)	(struct _j1GFX_CTRL *j1GFX_Ctrl, j1GFX_ROTATE rotate);
	void	(*DrawPixel)	(struct _j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, uint32 rgb);
	void	(*DrawVLine)	(struct _j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 h, uint32 rgb);
	void	(*DrawHLine)	(struct _j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 w, uint32 rgb);
	void	(*DrawFont)		(struct _j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, char c, j1GFX_FONT font, uint16 w, uint16 h, uint8 scale, uint32 fgRGB, uint32 bgRGB);
	void	(*FillRect)		(struct _j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 w, int16 h, uint32 rgb);
	
	void	(*Calibrate)	(struct _j1GFX_CTRL *j1GFX_Ctrl, bool preloaded);
	bool	(*Touched)		(void);
	bool	(*GetTouchXY)	(struct _j1GFX_CTRL *j1GFX_Ctrl, bool waitUp, int16 *x, int16 *y);

} j1GFX_HOOKS;
	
//---------------------------------------------------------------------
//	Screen Descriptor Language (SDL) DataTypes & Macros
//---------------------------------------------------------------------

#define	j1GFX_SDL_TEXT_OPTS_FILL	0x01
#define	j1GFX_SDL_TEXT_OPTS_FRAME	0x02
#define	j1GFX_SDL_TEXT_OPTS_VCENT	0x04
#define	j1GFX_SDL_TEXT_OPTS_HCENT	0x08
#define	j1GFX_SDL_TEXT_OPTS_BTN		(j1GFX_SDL_TEXT_OPTS_FILL | j1GFX_SDL_TEXT_OPTS_FRAME | j1GFX_SDL_TEXT_OPTS_VCENT | j1GFX_SDL_TEXT_OPTS_HCENT)
#define	j1GFX_SDL_TEXT_OPTS_ALL		0xff

typedef enum
{
	j1GFX_SDL_NONE,
#if j1GFX_PARAM_SDL_TEXT
	j1GFX_SDL_TEXT,
#endif
} j1GFX_SDL_TYPE;

typedef struct _j1GFX_SDL_ENTRY
{
	uint8			sdlID;
	j1GFX_SDL_TYPE	sdlType;
	void const		*sdlPtr;
} j1GFX_SDL_ENTRY;

#if j1GFX_PARAM_SDL_TEXT
typedef struct __j1GFX_SDL_Text
{
	uint8		opts;
	int16		x;
	int16		y;
	int16		w;
	int16		h;
	j1GFX_FONT	font;
	uint8		scale;
	uint32		fg;
	uint32		bg;
	char		*text;
} _j1GFX_SDL_Text;

#define j1GFX_SDL_Text(name, opts, x, y, w, h, font, scale, fg, bg, text) \
	_j1GFX_SDL_Text	const name = {  opts, x, y, w, h, font, scale, fg, bg, text };
#endif

//---------------------------------------------------------------------
//	function prototypes
//---------------------------------------------------------------------

void	j1GFX_SetRegion		(j1GFX_CTRL *j1GFX_Template, j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 w, int16 h);
void	j1GFX_DrawPixel		(j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, uint32 rgb);
void	j1GFX_DrawVLine		(j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 h, uint32 rgb);
void	j1GFX_DrawHLine		(j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 w, uint32 rgb);
void	j1GFX_DrawLine		(j1GFX_CTRL *j1GFX_Ctrl, int16 x0, int16 y0, int16 x1, int16 y1, uint32 rgb);
void	j1GFX_DrawRect		(j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 w, int16 h, uint32 rgb);
void	j1GFX_DrawCircle	(j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 r, uint32 rgb);
void	j1GFX_DrawTriangle	(j1GFX_CTRL *j1GFX_Ctrl, int16 x0, int16 y0, int16 x1, int16 y1, int16 x2, int16 y2, uint32 rgb);
void	j1GFX_GetFontDims	(j1GFX_FONT font, uint8 scale, uint16 *w, uint16 *h);
void	j1GFX_DrawFont		(j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, char c, j1GFX_FONT font, uint8 scale, uint32 fg, uint32 bg);
void	j1GFX_FillRect		(j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 w, int16 h, uint32 rgb);
void	j1GFX_FillRegion	(j1GFX_CTRL *j1GFX_Ctrl, uint32 rgb);
void	j1GFX_FillDisplay	(j1GFX_CTRL *j1GFX_Ctrl, uint32 rgb);
void	j1GFX_SetRotation	(j1GFX_CTRL *j1GFX_Ctrl, j1GFX_ROTATE rotate);
void	j1GFX_SetFontFgBg	(j1GFX_CTRL *j1GFX_Ctrl, j1GFX_FONT font, uint8 scale, uint32 fg, uint32 bg);
void	j1GFX_SetTextRowCol	(j1GFX_CTRL *j1GFX_Ctrl, uint8 row, uint8 col);
void	j1GFX_SetTextXY		(j1GFX_CTRL *j1GFX_Ctrl, uint16 x, uint16 y);
void	j1GFX_PutChar		(j1GFX_CTRL *j1GFX_Ctrl, char c);
void	j1GFX_PutString		(j1GFX_CTRL *j1GFX_Ctrl, char *string);

void	j1GFX_Text			(j1GFX_CTRL *j1GFX_Ctrl, uint8 opts, int16 x, int16 y, int16 w, int16 h, j1GFX_FONT font, uint8 scale, uint32 fg, uint32 bg, char *text);

void	j1GFX_SDL_Draw		(j1GFX_CTRL *j1GFX_Ctrl, j1GFX_SDL_ENTRY const *sdlTable, uint8 sdlCount);
void	j1GFX_SDL_HitTest	(j1GFX_CTRL *j1GFX_Ctrl, j1GFX_SDL_ENTRY const *sdlTable, uint8 sdlCount, j1GFX_SDL_ENTRY *hit);

#endif	// j1GFX_H

