//=====================================================================
//
//	j1HY28B_LCD.hh - driver file for HAOYU ILI9325C 2.8" TFT
//
//	version 1.1 - 15 feb 2014
//
//=====================================================================

//=====================================================================
//
// Copyright (c) 2015 Joshua 1 Systems Inc. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification, are
// permitted provided that the following conditions are met:
//
//    1. Redistributions of source code must retain the above copyright notice, this list of
//       conditions and the following disclaimer.
//
//    2. Redistributions in binary form must reproduce the above copyright notice, this list
//       of conditions and the following disclaimer in the documentation and/or other materials
//       provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY JOSHUA 1 SYSTEMS INC. "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
// ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
// ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation are those of the
// authors and should not be interpreted as representing official policies, either expressed
// or implied, of Joshua 1 Systems Inc.
//
//=====================================================================

#if !defined(j1HY28B_LCD_HH)
#define j1HY28B_LCD_HH
	
//#define	j1HY28B_LCD_PAR_MODE	1
//#define	j1HY28B_LCD_SPI_MODE	0
//#define	j1HY28B_SENDER			0
//#define	j1HY28B_RECEIVER		0
	
//#include "j1GFX.h"

//---------------------------------------------------------------------
//	FreeRTOS includes
//---------------------------------------------------------------------

//#include <FreeRTOS.h>
//#include <list.h>
//#include <queue.h>
//#include <semphr.h>
//#include <task.h>
//#include <timers.h>
	
//---------------------------------------------------------------------
//	defines for HY28B
//---------------------------------------------------------------------

#define	j1HY28B_LCD_DelayMS(ms)	CyDelay(ms)

#define j1HY28B_LCD_TFTWIDTH	240
#define j1HY28B_LCD_TFTHEIGHT	320

#define j1HY28B_LCD_START   (0x70)              /* Start byte for SPI transfer        */
#define j1HY28B_LCD_RD      (0x01)              /* WR bit 1 within start              */
#define	j1HY28B_LCD_WR      (0x00)              /* WR bit 0 within start              */
#define j1HY28B_LCD_DATA    (0x02)              /* RS bit 1 within start byte         */
#define j1HY28B_LCD_INDEX   (0x00)              /* RS bit 0 within start byte         */

#define	j1HY28B_LCD_EM_03_TRI		0x8000			/* tri mode									*/
#define	j1HY28B_LCD_EM_03_DFM		0x4000			/* dfm mode if tri = 1						*/
#define	j1HY28B_LCD_EM_03_BGR		0x1000			/* bgr=0 (RGB), bgr=1 (GRB)					*/
#define	j1HY28B_LCD_EM_03_ORG		0x0080			/* org=0 leave org, org=1 follow directions	*/
#define	j1HY28B_LCD_EM_03_ID1		0x0020			/* i/d1 - inc/dec bit 1						*/
#define	j1HY28B_LCD_EM_03_ID0		0x0010			/* i/d0 - inc/dec bit 0						*/
#define	j1HY28B_LCD_EM_03_AM		0x0008			/* am - addressing mode						*/

// control register masks

#define	j1HY28B_CTRL_RESET		0x01
#define	j1HY28B_CTRL_SS		0x02
#define j1HY28B_CTRL_TSS		0x04
#define	j1HY28B_CTRL_TCLK_LCLK	0x08
#define	j1HY28B_CTRL_RS		0x10
#define	j1HY28B_CTRL_RD		0x20
#define	j1HY28B_CTRL_WR		0x40
#define	j1HY28B_CTRL_PAR_SER	0x80

//---------------------------------------------------------------------
//	j1HY28B_LCD hardware ctrl
//---------------------------------------------------------------------

typedef struct _j1HY28B_LCD_CTRL
{
	uint8			dispType;					// display type (driver dependent)
												// not used for HY28B
												// but we need at least a 1 byte struct
												// for compatibility with other
												// drivers
	
} j1HY28B_LCD_CTRL;

//---------------------------------------------------------------------
//	function prototypes
//---------------------------------------------------------------------

void j1HY28B_LCD_SetRotation(j1GFX_CTRL *j1GFX_Ctrl, j1GFX_ROTATE rotate);
void j1HY28B_LCD_DrawPixel(j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, uint32 rgb);
void j1HY28B_LCD_DrawVLine(j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 h, uint32 rgb);
void j1HY28B_LCD_DrawHLine(j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 w, uint32 rgb);
void j1HY28B_LCD_DrawFont(j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16_t y, char c, j1GFX_FONT font, uint16 w, uint16 h, uint8 scale, uint32 fgRGB, uint32 bgRGB);
void j1HY28B_LCD_FillRect(j1GFX_CTRL *j1GFX_Ctrl, int16 x, int16 y, int16 w, int16 h, uint32 rgb);

#endif	// j1HY28B_LCD_HH
