/*******************************************************************************
* File Name: j1HY28B_iRS.c  
* Version 2.10
*
* Description:
*  This file contains API to enable firmware control of a Pins component.
*
* Note:
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "j1HY28B_iRS.h"

/* APIs are not generated for P15[7:6] on PSoC 5 */
#if !(CY_PSOC5A &&\
	 j1HY28B_iRS__PORT == 15 && ((j1HY28B_iRS__MASK & 0xC0) != 0))


/*******************************************************************************
* Function Name: j1HY28B_iRS_Write
********************************************************************************
*
* Summary:
*  Assign a new value to the digital port's data output register.  
*
* Parameters:  
*  prtValue:  The value to be assigned to the Digital Port. 
*
* Return: 
*  None
*  
*******************************************************************************/
void j1HY28B_iRS_Write(uint8 value) 
{
    uint8 staticBits = (j1HY28B_iRS_DR & (uint8)(~j1HY28B_iRS_MASK));
    j1HY28B_iRS_DR = staticBits | ((uint8)(value << j1HY28B_iRS_SHIFT) & j1HY28B_iRS_MASK);
}


/*******************************************************************************
* Function Name: j1HY28B_iRS_SetDriveMode
********************************************************************************
*
* Summary:
*  Change the drive mode on the pins of the port.
* 
* Parameters:  
*  mode:  Change the pins to one of the following drive modes.
*
*  j1HY28B_iRS_DM_STRONG     Strong Drive 
*  j1HY28B_iRS_DM_OD_HI      Open Drain, Drives High 
*  j1HY28B_iRS_DM_OD_LO      Open Drain, Drives Low 
*  j1HY28B_iRS_DM_RES_UP     Resistive Pull Up 
*  j1HY28B_iRS_DM_RES_DWN    Resistive Pull Down 
*  j1HY28B_iRS_DM_RES_UPDWN  Resistive Pull Up/Down 
*  j1HY28B_iRS_DM_DIG_HIZ    High Impedance Digital 
*  j1HY28B_iRS_DM_ALG_HIZ    High Impedance Analog 
*
* Return: 
*  None
*
*******************************************************************************/
void j1HY28B_iRS_SetDriveMode(uint8 mode) 
{
	CyPins_SetPinDriveMode(j1HY28B_iRS_0, mode);
}


/*******************************************************************************
* Function Name: j1HY28B_iRS_Read
********************************************************************************
*
* Summary:
*  Read the current value on the pins of the Digital Port in right justified 
*  form.
*
* Parameters:  
*  None
*
* Return: 
*  Returns the current value of the Digital Port as a right justified number
*  
* Note:
*  Macro j1HY28B_iRS_ReadPS calls this function. 
*  
*******************************************************************************/
uint8 j1HY28B_iRS_Read(void) 
{
    return (j1HY28B_iRS_PS & j1HY28B_iRS_MASK) >> j1HY28B_iRS_SHIFT;
}


/*******************************************************************************
* Function Name: j1HY28B_iRS_ReadDataReg
********************************************************************************
*
* Summary:
*  Read the current value assigned to a Digital Port's data output register
*
* Parameters:  
*  None 
*
* Return: 
*  Returns the current value assigned to the Digital Port's data output register
*  
*******************************************************************************/
uint8 j1HY28B_iRS_ReadDataReg(void) 
{
    return (j1HY28B_iRS_DR & j1HY28B_iRS_MASK) >> j1HY28B_iRS_SHIFT;
}


/* If Interrupts Are Enabled for this Pins component */ 
#if defined(j1HY28B_iRS_INTSTAT) 

    /*******************************************************************************
    * Function Name: j1HY28B_iRS_ClearInterrupt
    ********************************************************************************
    * Summary:
    *  Clears any active interrupts attached to port and returns the value of the 
    *  interrupt status register.
    *
    * Parameters:  
    *  None 
    *
    * Return: 
    *  Returns the value of the interrupt status register
    *  
    *******************************************************************************/
    uint8 j1HY28B_iRS_ClearInterrupt(void) 
    {
        return (j1HY28B_iRS_INTSTAT & j1HY28B_iRS_MASK) >> j1HY28B_iRS_SHIFT;
    }

#endif /* If Interrupts Are Enabled for this Pins component */ 

#endif /* CY_PSOC5A... */

    
/* [] END OF FILE */
